<?php
class GFQueriesUI {
	public static function join_by($array, $field) {
		$results = array();
		foreach ($array as $row) {
			$key = $row->$field;
			$key = $key == '' ? '(Unknown)' : $key;
			$found = false;
			for ($i = 0; $i<count($results);  $i++) {
				$result = $results[$i];
				if ($result['key'] == $key) {
					$result['amount'] += $row->amount;
					$result['count'] = ($result['count'] * $result['total'] + $row->count) / ($result['total']+1);
					$result['total']++;
					$results[$i] = $result;
					$found = true;
					break;
				}
			}
			if (!$found) {
				$results[] = array(
					'key' =>  $key,
					'amount' => $row->amount,
					'total' => 1,
					'count' => $row->count,
					'average' => $row->amount
				);
			}
		}
		foreach ($results as $result) {
			$result['average'] = $result['amount'] / $result['total'];
		}
		return $results;
	}
	public static function consolidate($array, $field, $amount = 10) {
		$results = array();
		$other = array(
			'key' =>  '(Other)',
			'amount' => 0,
			'total' => 0,
			'count' => 0,
			'average' => 0
		);
		foreach ($array as $value) {
			if ($value[$field] < $amount) {
				//$other['average'] = ($other['average']*$other['total'] + $value['average']*$value['total']) / ($other['total'] + $value['total'])
				$other['amount'] += $value['amount'];
				$other['count'] = ($other['count']*$other['total'] + $value['count']*$value['total']) / ($other['total'] + $value['total']);
				$other['total'] += $value['total'];
				$other['average'] = $other['amount'] / $other['total']; //Rather than rolling average above, just calculate a new absolute average
			} else {
				$results[] = $value;
			}
		}
		$results[] = $other;
		return $results;
	}
	public static function number_format($number, $decimals = 0) {
		return number_format(self::check_number($number), $decimals);
	}
	public static function check_number($number) {
		return is_nan($number) ? 0 : $number;
	}
	public static function get_oneoff($transactions) {
		$results = array();
		foreach($transactions as $row) {
			if (!$row->recurring) {
				$results[] = $row;
			}
		}
		return $results;
	}
	public static function get_newsubscriptions($transactions) {
		$results = array();
		foreach($transactions as $row) {
			if ($row->recurring && $row->first_payment) {
				$results[] = $row;
			}
		}
		return $results;
	}
	public static function get_recurring($transactions) {
		$results = array();
		foreach($transactions as $row) {
			if ($row->recurring && !$row->first_payment) {
				$results[] = $row;
			}
		}
		return $results;
	}
	public static function by_source($transactions, $joinfield, $cutoff = 10, $outputfield = 'total') {
		$result = self::join_by($transactions, $joinfield);
		//Consolidate always by number of payments, as that makes more sense than average payment, and amount would need to be calculated differently, like < $100 rather than < 10 (payments)
		//$result = self::consolidate($result, $outputfield, $cutoff);
		$result = self::consolidate($result, 'total', $cutoff);
		array_multisort(array_column($result, $outputfield), SORT_DESC, $result);
		return $result;
	}
	public static function pie_data($transactions, $joinfield, $cutoff = 10, $outputfield = 'total') {
		$transactions = self::by_source($transactions, $joinfield, $cutoff, $outputfield);
		$js = "[";
		foreach ($transactions as $value) {
			$js .= "['".$value['key']."', ".self::check_number($value[$outputfield])."],";
		}
		$js .= "
		];";
		return $js;
	}
	public static function output_pie($title, $id, $transactions, $joinfield, $cutoff = 10, $outputfield = 'total', $displayvalue = false) {
		return "var info = ".self::pie_data($transactions, $joinfield, $cutoff, $outputfield)."
			drawPie('$title', '$id', info, ".$displayvalue.");";
	}
	public static function output_table($transactions, $field, $cutoff = 10) {
		$transactions = self::by_source($transactions, $field, $cutoff);
		$html = '';
		foreach ($transactions as $value) {
			$average = $value['amount']/$value['total'];
			$html .= '<tr><td>'.self::table_name($value['key'], $field).'</td>
				<td data-sort-value="'.$value['total'].'">'.$value['total'].'</td>
				<td data-sort-value="'.$value['amount'].'">$'.self::number_format($value['amount']).'</td>
				<td data-sort-value="'.self::check_number($average).'">$'.self::number_format($average,2).'</td></tr>';
		}
		return $html;
	}
	public static function table_name($key, $field) {
		if ($key!='(Other)' && $key!='(Unknown)' && ($field=='page' || $field=='description')) {
			//if ($field == 'page') {
			//	return '<a href="'.$key.'" target="_blank">'.$key.'</a>';
			//} else {
				return '<a href="'.admin_url('admin.php?page=gf-queries&'.($field=='page'?'path':'source').'='.urlencode($key).
				($_GET['period']?'&period='.$_GET['period']:'').
				($_GET['start']?'&start='.$_GET['start']:'').
				($_GET['end']?'&end='.$_GET['end']:'')).
				'">'.$key.'</a>';
			//}
		} else {
			return $key;
		}
	}
	public static function output_subscriptions($transactions, $field, $cutoff = 10) {
		$transactions = self::by_source($transactions, $field, $cutoff);
		$html = '';
		foreach ($transactions as $value) {
			$average = $value['amount']/$value['total'];
			$customervalue = $average * $value['count'];
			$html .= '<tr><td>'.self::table_name($value['key'], $field).'</td>
				<td data-sort-value="'.$value['total'].'">'.self::number_format($value['total']).'</td>
				<td data-sort-value="'.$value['count'].'">'.self::number_format($value['count'],1).'</td>
				<td data-sort-value="'.self::check_number($average).'">$'.self::number_format($average,2).'</td>
				<td data-sort-value="'.self::check_number($customervalue).'">$'.self::number_format($customervalue).'</td></tr>';
		}
		return $html;
	}
	public static function get_transactions($start, $end = null) {
		$transactions = GFQueries::get_transactions($start, $end, true);
		for ($i=0; $i<count($transactions); $i++) {
			$url = parse_url($transactions[$i]->source_url);
			$transactions[$i]->page = rtrim($url['path'], '/'); //Remove trailing slashes as some do and some don't
			if ($transactions[$i]->description == '') {
				$transactions[$i]->description = $transactions[$i]->page;
			}
		}
		return $transactions;
	}
	public static function history($array, $start, $end, $internal = 'month', $filter = null, $value = null) {
		$results = array();
		$i=0;
		do {
			$date1 = new DateTime();
			$date1 = $date1->setTimestamp($start);
			$date2 = new DateTime();
			$date2 = $date2->setTimestamp($start);
			if ($i != 0) {
				$date1 = $date1->modify('+'.$i.' '.$internal);
			}
			$date2 = $date2->modify('+'.($i+1).' '.$internal);
			$format = $internal == 'month' ? "M 'y" : 'j M';

			$results[] = array(
				'key' => date($format, $date1->getTimestamp()),
				'amountone' => 0,
				'totalone' => 0,
				'amountrec' => 0,
				'totalrec' => 0,
				'amountnew' => 0,
				'totalnew' => 0,
				'start' => $date1->getTimestamp(),
				'end' => $date2->getTimestamp(),
				'startdate' => date('Y-m-d', $date1->getTimestamp()),
				'enddate' => date('Y-m-d', $date2->getTimestamp()),
			);
			$i++;
		} while ($date2->getTimestamp() < $end);

		foreach ($array as $row) {
			for ($i=0; $i<count($results); $i++) {
				$result = $results[$i];
				if (($row->date >= $result['start'] && $row->date < $result['end']) &&
					(!$filter || !$value || $row->$filter == $value)) {
					if ($row->recurring && $row->first_payment) {
						$result['amountnew'] += $row->amount;
						$result['totalnew'] += 1;
					} elseif ($row->recurring) {
						$result['amountrec'] += $row->amount;
						$result['totalrec'] += 1;
					} else {
						$result['amountone'] += $row->amount;
						$result['totalone'] += 1;
					}
					$results[$i] = $result;
					break;
				}
			}
		}
		return $results;
	}
	public static function subscription_length($array) {
		$results = array();
		foreach ($array as $row) {
			if ($row->recurring) {
				$found = false;
				for ($i=0; $i<count($results); $i++) {
					$result = $results[$i];
					if ($result->entry_id == $row->entry_id) {
						//$result->amount += $row->amount;
						//We don't overwite count like join_by because we want the absolute count
						//$result['count'] = ($result['count'] * $result['total'] + $row->count) / ($result['total']+1);
						//$result->total++;
						//$results[$i] = $result;
						$found = true;
						break;
					}
				}
				if (!$found) {
					$results[] = $row;
				}
			}
		}
		return $results;
	}
	public static function transactions_page() {
		$period = $_GET['period'] ? $_GET['period'] : (isset($_GET['start']) ? '' : 'year');
		$source = $_GET['source'];
		$path = $_GET['path'];
		echo '<h1>Analytics | <a href="'.admin_url('admin.php?page=gf-queries').'">Year</a> | <a href="'.admin_url('admin.php?page=gf-queries&period=month').'">Month</a> | <a href="'.admin_url('admin.php?page=gf-queries&period=week').'">Week</a></h1>';
		$now = new DateTime('now');
		$end = (new DateTime('now'))->getTimestamp();
		$title = $source ? $source.' - ' : ($path ? $path.' ' : '');
		if ($period == 'year') {
			$sourcegroup = 0;
			$pagegroup = 0;
			$diff = '-13 month';
			$start = $now->modify($diff);
			$start = strtotime(date('Y-m-01', $start->getTimestamp()));
			$internal = 'month';
			$title .= 'Last Year';
		} elseif ($period == 'week') {
			$sourcegroup = 0;
			$pagegroup = 0;
			$diff = '-1 week';
			$start = $now->modify($diff);
			$start = strtotime(date('Y-m-d H:i:s', $start->getTimestamp()));
			$internal = 'day';
			$title .= 'Last Week';
		} else {
			$sourcegroup = 0;
			$pagegroup = 0;
			if ($period == 'month') {
				$diff = '-1 month';
				$start = $now->modify($diff);
				$start = strtotime(date('Y-m-d H:i:s', $start->getTimestamp()));
				$title .= 'Last Month';
			} else {
				$start = strtotime($_GET['start']);
				$end = strtotime($_GET['end']);
				$titlestart = new DateTime();
				$titlestart = $titlestart->setTimestamp($start);
				$titleend =  new DateTime();
				$titleend = $titleend->setTimestamp($end);
				$titleend = $titleend->modify('-1 day');
				if ($titlestart->getTimestamp() == $titleend->getTimestamp()) {
					$title .= date('j F Y', $titlestart->getTimestamp());
				} else {
					$title .= date('F Y', $titlestart->getTimestamp());
				}
			}
			$internal = 'day';
		}
		$transactions = self::get_transactions($start, $end);
		$filter = $source ? 'description' : 'page';
		$filtervalue = $source ? $source : $path;
		$history = self::history($transactions, $start, $end, $internal, $filter, $filtervalue);
		$oneoff = self::get_oneoff($transactions);
		$newsub = self::get_newsubscriptions($transactions);
		$recurr = self::get_recurring($transactions);

		$subscriptionlength = self::subscription_length($transactions);

		echo '<div id="gf_queries">
		<style>
		h2 a {
			cursor: pointer;
		}
		.gfq_image {
			width: 100%;
		}
		.gfq_chart_wrap {
			position: absolute;
			top: 0;
			bottom: 0;
			width: 100%;
			height: 100%;
		}
		.gfq_cell_wrap {
			position: relative;
			width: 100%;
			height: 100%;
		}
		.gfq_chart {
			/*min-height: 320px;*/
			height: 100%;
			width: 100%;
		}
		.gfq_table {
			width: 100%;
		}
		.gfq_table th {
			font-weight: bold;
			text-align: right;
		}
		.gfq_table td.gfq_nowrap {
			white-space: nowrap;
		}
		.gfq_table td {
			vertical-align: top;
			text-align: right;
		}
		.gfq_table2 td {
			width: 50%;
		}
		.gfq_table3 td {
			width: 33%;
		}
		.gfq_table2 table td,
		.gfq_table3 table td {
			width: auto;
		}
		.gfq_table .gfq_left th,
		.gfq_table th:first-child,
		.gfq_table td:first-child {
			text-align: left;
		}
		</style>';
		echo '<script type="text/javascript" src="'.plugin_dir_url(__FILE__).'js/stupidtable.min.js"></script>';
		echo '<script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>';
		echo "<script>
			google.charts.load('current', {'packages':['corechart']});
			google.charts.setOnLoadCallback(drawChart);
			var history = chartData();
			var sourceone = chartData();
			var sourcenew = chartData();
			var sourcerec = chartData();
			var pageone = chartData();
			var pagenew = chartData();
			var pagerec = chartData();
			var queriesurl = '".admin_url('admin.php?page=gf-queries')."';
			function chartData() {
				return {
					chart: null,
					payments: null,
					amount: null,
					average: null,
					legend: null
				};
			}
			function init() {
				history.legend = [";
				foreach ($history as $value) {
					echo "{start: '".$value['startdate']."', end: '".$value['enddate']."'},";
				}
				echo "];
				history.payments = google.visualization.arrayToDataTable([
					['Period', 'One Time', 'New Subscriptions', 'Recurring', { role: 'annotation' }],
					";
					foreach ($history as $value) {
						echo "[\"".$value['key']."\", ".$value['totalone'].", ".$value['totalnew'].", ".$value['totalrec'].", ''],";
					}
					echo "
				]);
				history.amount = google.visualization.arrayToDataTable([
					['Period', 'One Time', 'New Subscriptions', 'Recurring', { role: 'annotation' }],
					";
					foreach ($history as $value) {
						echo "[\"".$value['key']."\", ".$value['amountone'].", ".$value['amountnew'].", ".$value['amountrec'].", ''],";
					}
					echo "
				]);
				history.average = google.visualization.arrayToDataTable([
					['Period', 'One Time', 'New Subscriptions', 'Recurring', { role: 'annotation' }],
					";
					foreach ($history as $value) {
						echo "[\"".$value['key']."\", ".round(self::check_number($value['amountone']/$value['totalone']),2).", ".round(self::check_number($value['amountnew']/$value['totalnew']),2).", ".round(self::check_number($value['amountrec']/$value['totalrec']),2).", ''],";
					}
					echo "
				]);
				sourceone.payments = ".self::pie_data($oneoff,'description',$sourcegroup)."
				sourcenew.payments = ".self::pie_data($newsub,'description',$sourcegroup)."
				sourcerec.payments = ".self::pie_data($recurr,'description',$sourcegroup)."
				pageone.payments = ".self::pie_data($oneoff,'page',$pagegroup)."
				pagenew.payments = ".self::pie_data($newsub,'page',$pagegroup)."
				pagerec.payments = ".self::pie_data($recurr,'page',$pagegroup)."

				sourceone.amount = ".self::pie_data($oneoff,'description',$sourcegroup,'amount')."
				sourcenew.amount = ".self::pie_data($newsub,'description',$sourcegroup,'amount')."
				sourcerec.amount = ".self::pie_data($recurr,'description',$sourcegroup,'amount')."
				pageone.amount = ".self::pie_data($oneoff,'page',$pagegroup,'amount')."
				pagenew.amount = ".self::pie_data($newsub,'page',$pagegroup,'amount')."
				pagerec.amount = ".self::pie_data($recurr,'page',$pagegroup,'amount')."

				sourceone.average = ".self::pie_data($oneoff,'description',$sourcegroup,'average')."
				sourcenew.average = ".self::pie_data($newsub,'description',$sourcegroup,'average')."
				sourcerec.average = ".self::pie_data($recurr,'description',$sourcegroup,'average')."
				pageone.average = ".self::pie_data($oneoff,'page',$pagegroup,'average')."
				pagenew.average = ".self::pie_data($newsub,'page',$pagegroup,'average')."
				pagerec.average = ".self::pie_data($recurr,'page',$pagegroup,'average')."
			}
			function drawPayments() {
				drawHistory(history.payments);
				drawSource(sourceone.payments, sourcenew.payments, sourcerec.payments, '(Payments)');
				drawPage(pageone.payments, pagenew.payments, pagerec.payments, '(Payments)');
			}
			function drawAmount() {
				drawHistory(history.amount);
				drawSource(sourceone.amount, sourcenew.amount, sourcerec.amount, '(Amount)');
				drawPage(pageone.amount, pagenew.amount, pagerec.amount, '(Amount)');
			}
			function drawAverage() {
				drawHistory(history.average);
				drawSource(sourceone.average, sourcenew.average, sourcerec.average, '(Average)');
				drawPage(pageone.average, pagenew.average, pagerec.average, '(Average)');
			}
			function drawChart() {
				init();
				drawPayments();
				drawOther();
			}
			function drawHistory(data) {
				var options = {
					width: '100%',
					height: '100%',
					legend: { position: 'top', maxLines: 3 },
					bar: { groupWidth: '75%' },
					isStacked: true
				};
				history.chart = new google.visualization.ColumnChart(document.getElementById('gfq_chart_recurring'));
				history.chart.draw(data, options);
				google.visualization.events.addListener(history.chart, 'select', historyClick);
			}
			function queryParams(name, url) {
				//var url = new URL(url || location.href);
				//return url.searchParams.get(name);

				if (!url) url = window.location.href;
				name = name.replace(/[\[\]]/g, '\\$&');
				var regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)'),
					results = regex.exec(url);
				if (!results) return null;
				if (!results[2]) return '';
				return decodeURIComponent(results[2].replace(/\+/g, ' '));
			}
			function historyClick(e) {
				var selection = history.chart.getSelection();
				var date = history.legend[selection[0].row];
				var source = queryParams('source');
				var path = queryParams('path');
				location.href = queriesurl + (source?'&source='+encodeURIComponent(source):'') + (path?'&path='+encodeURIComponent(path):'') + '&start=' + date.start + '&end=' + date.end;
			}
			function drawSource(dataone, datanew, datarec, suffix) {
				drawPie('One Time '+suffix, 'gfq_chart_source_one', dataone, false, 'source');
				drawPie('New Subscriptions', 'gfq_chart_source_new', datanew, false, 'source');
				drawPie('Recurring', 'gfq_chart_source_rec', datarec, false, 'source');
			}
			function drawPage(dataone, datanew, datarec, suffix) {
				drawPie('One Time '+suffix, 'gfq_chart_page_one', dataone, false, 'path');
				drawPie('New Subscriptions '+suffix, 'gfq_chart_page_new', datanew, false, 'path');
				drawPie('Recurring '+suffix, 'gfq_chart_page_rec', datarec, false, 'path');
			}
			function drawOther() {
				drawType();
				drawLengthSource();
				drawLengthPage()
			}
			function drawType() {
				".self::output_pie('Payment Method','gfq_chart_method',$transactions,'payment_method',0)."
			}
			function drawLengthSource() {
				".self::output_pie('Subscription Length By Source','gfq_chart_length_source',$subscriptionlength,'description',0,'count',true)."
			}
			function drawLengthPage() {
				".self::output_pie('Subscription Length by Page','gfq_chart_length_page',$subscriptionlength,'page',0,'count',true)."
			}
			function pieClick(filter, value) {
				var period = queryParams('period');
				var start = queryParams('start');
				var end = queryParams('end');
				location.href = queriesurl + '&'+filter+'=' + encodeURIComponent(value) + (period?'&period='+period:'') + (start?'&start='+start:'') + (end?'&end='+end:'');
			}
			function drawPie(title, id, info, value, filter) {
				var data = new google.visualization.DataTable();
				data.addColumn('string', 'Source');
				data.addColumn('number', 'Payments');
				data.addRows(info);
				var options = {'title':title, 'width':'100%', 'height':'100%', pieSliceText: value?'value':'percentage'};
				var chart = new google.visualization.PieChart(document.getElementById(id));
				chart.draw(data, options);
				google.visualization.events.addListener(chart, 'select', function(e) {
					var selection = chart.getSelection();
					var filtervalue = info[selection[0].row][0];
					if (filtervalue != '(Other)' && filtervalue != '(Unknown)') {
						//console.log(filtervalue);
						pieClick(filter, filtervalue);
					}
				});
			}
			jQuery(document).ready(function(e) {
				jQuery('.gfq_breadown_table').stupidtable();
				jQuery('#gfq_history_table').stupidtable();
				if (jQuery('#gfq_transaction_table').length > 0) {
					jQuery('#gfq_transaction_table').stupidtable();
				}
			});
			</script>";
		
		$image = plugin_dir_url(__FILE__).'/images/6by4.png';
		echo '<div>
			<h2>'.$title.' | <a onclick="drawPayments()">Payments</a> | <a onclick="drawAmount()">Amount</a> | <a onclick="drawAverage()">Average</a></h2>
			<table class="gfq_table gfq_table2"><tr>
			<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_bar" id="gfq_chart_recurring"></div></div></div></td>
			<td><table class="widefat striped" id="gfq_history_table"><thead>
					<tr class="gfq_left"><th></th><th colspan="3">One Off</th><th colspan="3">New Subscriptions</th><th colspan="3">Recurring</th></tr>
					<tr><th data-sort="int">Period</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr>
				</thead>
				<tbody>';
				$history = array_reverse($history);
				foreach ($history as $value) {
					echo '<tr><td data-sort-value="'.$value['start'],'">'.$value['key'].'</td>
						<td data-sort-value="'.$value['totalone'].'">'.$value['totalone'].'</td><td data-sort-value="'.$value['amountone'].'">$'.self::number_format($value['amountone']).'</td><td data-sort-value="'.self::check_number($value['amountone']/$value['totalone']).'">$'.self::number_format($value['amountone']/$value['totalone'],2).'</td>
						<td data-sort-value="'.$value['totalnew'].'">'.$value['totalnew'].'</td><td data-sort-value="'.$value['amountnew'].'">$'.self::number_format($value['amountnew']).'</td><td data-sort-value="'.self::check_number($value['amountnew']/$value['totalnew']).'">$'.self::number_format($value['amountnew']/$value['totalnew'],2).'</td>
						<td data-sort-value="'.$value['totalrec'].'">'.$value['totalrec'].'</td><td data-sort-value="'.$value['amountrec'].'">$'.self::number_format($value['amountrec']).'</td><td data-sort-value="'.self::check_number($value['amountrec']/$value['totalrec']).'">$'.self::number_format($value['amountrec']/$value['totalrec'],2).'</td>
					</tr>';
				}
			echo '</tbody></table></td>
		</tr></table></div>';
		
		echo '<div>
			<h2>By Source | <a onclick="drawPayments()">Payments</a> | <a onclick="drawAmount()">Amount</a> | <a onclick="drawAverage()">Average</a></h2>
			<table class="gfq_table gfq_table3">
				<tr>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_source_one"></div></div></div></td>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_source_new"></div></div></div></td>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_source_rec"></div></div></div></td>
				</tr>
				<tr>
					<td><table class="widefat striped gfq_breadown_table">
					<thead><tr><th data-sort="string">Source</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr></thead>
					<tbody>'.self::output_table($oneoff,'description',$sourcegroup).'</tbody>
					</table></td>

					<td><table class="widefat striped gfq_breadown_table">
					<thead><tr><th data-sort="string">Source</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr></thead>
					<tbody>'.self::output_table($newsub,'description',$sourcegroup).'</tbody>
					</table></td>

					<td><table class="widefat striped gfq_breadown_table">
					<thead><tr><th data-sort="string">Source</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr></thead>
					<tbody>'.self::output_table($recurr,'description',$sourcegroup).'</tbody>
					</table></td>
				</tr>
			</table>
			</div>';

		echo '<div>
			<h2>By Page | <a onclick="drawPayments()">Payments</a> | <a onclick="drawAmount()">Amount</a> | <a onclick="drawAverage()">Average</a></h2>
			<table class="gfq_table gfq_table3">
				<tr>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_page_one"></div></div></div></td>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_page_new"></div></div></div></td>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_page_rec"></div></div></div></td>
				</tr>
				<tr>
					<td><table class="widefat striped gfq_breadown_table">
					<thead><tr><th data-sort="string">Page</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr></thead>
					<tbody>'.self::output_table($oneoff,'page',$pagegroup).'</tbody>
					</table></td>

					<td><table class="widefat striped gfq_breadown_table">
					<thead><tr><th data-sort="string">Page</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr></thead>
					<tbody>'.self::output_table($newsub,'page',$pagegroup).'</tbody>
					</table></td>

					<td><table class="widefat striped gfq_breadown_table">
					<thead><tr><th data-sort="string">Page</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr></thead>
					<tbody>'.self::output_table($recurr,'page',$pagegroup).'</tbody>
					</table></td>
				</tr>
			</table>
			</div>';

		
		echo '<div>
			<h2>Other Analytics</h2>
			<table class="gfq_table gfq_table3">
				<tr>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_method"></div></div></div></td>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_length_source"></div></div></div></td>
					<td><div class="gfq_cell_wrap"><img src="'.$image.'" class="gfq_image" /><div class="gfq_chart_wrap"><div class="gfq_chart gfq_chart_pie" id="gfq_chart_length_page"></div></div></div></td>
				</tr>
				<tr>
					<td><table class="widefat striped gfq_breadown_table">
					<thead><tr><th data-sort="string">Page</th><th data-sort="int">Payments</th><th data-sort="int">Amount</th><th data-sort="int">Average</th></tr></thead>
					<tbody>'.self::output_table($transactions,'payment_method',0).'</tbody>
					</table></td>

					<td><table class="widefat striped gfq_breadown_table">
					<thead>
						<tr class="gfq_left"><th></th><th></th><th colspan="3">Average</th></tr>
						<tr><th data-sort="string">Source</th><th data-sort="int">Subscribers</th><th data-sort="int">Length</th><th data-sort="int">Amount</th><th data-sort="int">Value</th></tr>
					</thead>
					<tbody>'.self::output_subscriptions($subscriptionlength,'description',0).'</tbody>
					</table></td>

					<td><table class="widefat striped gfq_breadown_table">
					<thead>
						<tr class="gfq_left"><th></th><th></th><th colspan="3">Average</th></tr>
						<tr><th data-sort="string">Page</th><th data-sort="int">Subscribers</th><th data-sort="int">Length</th><th data-sort="int">Amount</th><th data-sort="int">Value</th></tr>
					</thead>
					<tbody>'.self::output_subscriptions($subscriptionlength,'page',0).'</tbody>
					</table></td>
				</tr>
			</table>
			</div>';
		
		if ($period != 'year' || $filtervalue) {
			echo '<div>
			<h2>All Payments</h2>
				<table class="widefat striped" id="gfq_transaction_table">
				<thead><tr><th data-sort="int">Date</th><th data-sort="string">Name</th><th data-sort="string">Email</th><th data-sort="int">Amount</th><th data-sort="string">Type</th><th data-sort="string">Method</th><th data-sort="string">Description</th><th data-sort="string">Page</th></tr></thead>
				<tbody>';
				foreach ($transactions as $row) {
					if ($period != 'year' || $row->$filter == $filtervalue) {
						if ($row->recurring && $row->first_payment) {
							$type = 'New Subscription';
						} elseif ($row->recurring) {
							$type = 'Recurring';
						} else {
							$type = 'One Off';
						}
						//transaction_id
						//subscription_id
						echo '<tr><td class="gfq_nowrap" data-sort-value="'.$row->date.'"><a href="'.admin_url('admin.php?page=gf_entries&view=entry&id='.$row->form_id.'&lid='.$row->entry_id).'" target="_blank">'.date("Y-m-d", $row->date).' at '.date("h:i", $row->date).'</a></td>
							<td>'.$row->firstname.' '.$row->lastname.'</td><td>'.$row->email.'</td>
							<td data-sort-value="'.$row->amount.'">$'.self::number_format($row->amount).'</td><td>'.$type.'</td><td>'.$row->payment_method.'</td><td><a href="'.admin_url('admin.php?page=gf_entries&id='.$row->form_id).'" target="_blank">'.$row->description.'</a></td><td><a href="'.$row->page.'" target="_blank">'.$row->page.'</a></td></tr>';
					}
				}
				echo '</tbody></table>
			</div>';	
		}
		
		echo '</div>';
	}
}