<?php
/*
Plugin Name: GF Queries
Plugin URI: https://wordpress.org/plugins/gf-queries
Version: 1.2.3
Description: Some additional queries for Gravity Forms like getting recurring payment notifications
Author: James Low
Author URI: http://jameslow.com
*/

require_once 'gf-queries-forms.php';

class GFQueries {
	static $forms;
	static $fields;
	static $integers;
	static $floats;
	static $booleans;
	public static $PREFIX = 'gf-queries';
	public static $GROUP = 'gf-queries-settings-group';
	
	public static function get_form($form_id) {
		self::$forms = self::$forms == null ? array() : self::$forms;
		if (!isset(self::$forms[$form_id])) {
			self::$forms[$form_id] = RGFormsModel::get_form_meta($form_id);
		}
		if (self::$forms[$form_id] != null) {
			$meta = self::$forms[$form_id];
			if ($meta['gf-queries']) {
				return $meta['gf-queries'];
			}
		}
	}
	public static function fields() {
		if (self::$fields == null) {
			self::$fields = array('firstname', 'lastname', 'email', 'phone', 'address1', 'address2', 'city', 'state', 'zip', 'country', 'description', 'total', 'shipping', 'quantity', 'product');
		}
		return self::$fields;
	}
	public static function integers() {
		if (self::$integers == null) {
			self::$integers = array('form_id', 'entry_id', 'row_id', 'date', 'recurring', 'created');
		}
		return self::$integers;
	}
	public static function floats() {
		if (self::$floats == null) {
			self::$floats = array('original_amount', 'amount');
		}
		return self::$floats;
	}
	public static function booleans() {
		if (self::$booleans == null) {
			self::$booleans = array('recurring');
		}
		return self::$booleans;
	}
	public static function add_hooks() {
		add_action( 'rest_api_init', function () {
			register_rest_route( 'gf-queries/v1', 'transactions', array(
				'methods' => 'GET,POST',
				'callback' => array('GFQueries', 'transactions'),
				'args' => array()
			));
		});
		add_action('gform_loaded', array('GFQueries', 'gform_loaded'), 5);
		add_filter('gform_addon_navigation', array('GFQueries', 'create_menu' ));
	}
	public static function create_menu( $menus ) {
		$menus[] = array( 'name' => 'gf-queries', 'label' => __( 'Analytics' ), 'callback' =>  array('GFQueries', 'transactions_page') );
		return $menus;
	}
	public static function transactions_page() {
		require_once 'gf-queries-ui.php';
		GFQueriesUI::transactions_page();
	}
	public static function gform_loaded() {
		GFAddOn::register('GFQueriesAddon');
	}
	public static function checkapikey() {
		return self::apikey() == $_GET['apikey'];
	}
	public static function apikey() {
		$addon = GFQueriesAddon::get_instance();
		$settings = $addon->get_plugin_settings();
		$apikey = rgar($settings, 'gf_queries_apikey');
		if (!$apikey) {
			$apikey = md5(wp_salt() . time());
		}
		return $apikey;
	}
	public static function transactions() {
		if (self::checkapikey()) {
			return self::get_transactions($_GET['from'], $_GET['to']);
		} else {
			return new WP_Error('gf-queries-apikey', 'Invalid Api Key', array('status' => 200 ));
		}
	}
	public static function map_meta($t) {
		$entry = GFAPI::get_entry($t->entry_id);
		$map = self::get_form($t->form_id);
		$fields = self::fields();
		foreach ($fields as $field) {
			$fieldname = 'fields_'.$field;
			$t->$field = $map[$fieldname] ? $entry[$map[$fieldname]] : '';
		}
		return $t;
	}
	public static function get_transactions($from, $to = null, $analytics = false) {
		$transactions = self::query_transactions($from, $to);
		$results = array();
		foreach ($transactions as $t) {
			$form = self::get_form($t->form_id);
			if ($form && (($form['enabled'] && !$analytics) || ($form['analytics'] && $analytics))) {
				$t = self::map_meta($t);
				$integers = self::integers();
				foreach ($integers as $integer) {
					$t->$integer = (int) $t->$integer;
				}
				$floats = self::floats();
				foreach ($floats as $float) {
					$t->$float = (float) $t->$float;
				}
				$booleans = self::booleans();
				foreach ($booleans as $boolean) {
					$t->$boolean = (bool) $t->$boolean;
				}
				$results[] = $t;
			}
		}
		return $results;
	}
	public static function query_transactions($from, $to = null) {
		global $wpdb;
		$prefix = $wpdb->prefix;
		$fromsql = " AND t.date_created >= '" . date('Y-m-d H:i:s', $from) . "' ";
		$tosql = $to ? " AND t.date_created < '" . date('Y-m-d H:i:s', $to) . "' " : '';
		$sql = "
		SELECT 
			e.id as 'entry_id', e.form_id, e.ip, e.source_url, e.currency, e.payment_method, e.payment_amount as 'original_amount', e.transaction_id as 'original_id', UNIX_TIMESTAMP(e.date_created) as 'created',
			t.transaction_id, t.subscription_id, t.amount, t.transaction_type, t.id as 'row_id', UNIX_TIMESTAMP(t.date_created) as 'date', NOT(t.is_recurring) as 'first_payment', NOT(IFNULL(t.subscription_id,'') = '') as recurring, COUNT(t2.lead_id) as 'count'
			FROM ".$prefix."gf_entry e, ".$prefix."gf_addon_payment_transaction t, ".$prefix."gf_addon_payment_transaction t2
			WHERE e.id = t.lead_id AND e.status = 'active' AND t.transaction_type = 'payment' AND t.lead_id = t2.lead_id
			$fromsql $tosql
			GROUP BY t.transaction_id
			ORDER BY t.date_created DESC";
		/*$sql = "
			SELECT 
				e.id as 'entry_id', e.form_id, e.ip, e.source_url, e.currency, e.payment_method, e.payment_amount as 'original_amount', e.transaction_id as 'original_id', UNIX_TIMESTAMP(e.date_created) as 'created',
				t.transaction_id, t.subscription_id, t.amount, t.transaction_type, t.id as 'row_id', UNIX_TIMESTAMP(t.date_created) as 'date', NOT(t.is_recurring) as 'first_payment', NOT(IFNULL(t.subscription_id,'') = '') as recurring
				FROM ".$prefix."gf_entry e, ".$prefix."gf_addon_payment_transaction t
				WHERE e.id = t.lead_id AND e.status = 'active' AND t.transaction_type = 'payment'
				$fromsql $tosql
				ORDER BY t.date_created DESC";*/
		return $wpdb->get_results($sql);
	}
	/*public static function options_page() { ?>
	<div class="wrap">		
			<h1>GF Queries</h1>
			<form method="post" action="options.php">
			<?php settings_fields(self::$GROUP); ?>
			<?php do_settings_sections(self::$GROUP); ?>
			
			<table class="form-table">
				<tr valign="top">
					<th scope="row">API Key</th>
					<td><input placeholder="'API Key" type="text" name="gf-queries-apikey" value="<?php echo esc_attr(self::option('apikey')); ?>" /></td>
				</tr>
			</table>
			<?php submit_button(); ?>
			</form>
		</div>
	<?php }
	public static function option($id) {
		return get_option(self::$PREFIX.'-'.$id);
	}*/
}
GFQueries::add_hooks();